package goki.stats;

import goki.stats.handlers.PacketHandler;
import goki.stats.lib.Helper;
import goki.stats.lib.Reference;
import goki.stats.stats.Stat;
import goki.stats.stats.StatTreasureFinder;

import java.util.List;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.util.DamageSource;
import net.minecraftforge.event.ForgeSubscribe;
import net.minecraftforge.event.entity.EntityJoinWorldEvent;
import net.minecraftforge.event.entity.living.LivingDeathEvent;
import net.minecraftforge.event.entity.living.LivingEvent.LivingJumpEvent;
import net.minecraftforge.event.entity.living.LivingFallEvent;
import net.minecraftforge.event.entity.living.LivingHurtEvent;
import net.minecraftforge.event.entity.player.PlayerEvent;
import net.minecraftforge.event.world.BlockEvent.HarvestDropsEvent;
import cpw.mods.fml.common.network.PacketDispatcher;

public class CommonEvents
{
	@ForgeSubscribe
	public void harvestBlock(HarvestDropsEvent event)
	{
		EntityPlayer player = event.harvester;
		if (player != null)
		{
			if (Stat.getPlayersStatLevel(player, Stat.STAT_TREASURE_FINDER) > 0)
			{
				Random random = player.getRNG();
				List<ItemStack> items = ((StatTreasureFinder) Stat.STAT_TREASURE_FINDER).getApplicableItemStackList(event.block.blockID, event.blockMetadata, Stat.STAT_TREASURE_FINDER.getPlayersStatLevel(player));
				List<Integer> chances = ((StatTreasureFinder) Stat.STAT_TREASURE_FINDER).getApplicableChanceList(event.block.blockID, event.blockMetadata, Stat.STAT_TREASURE_FINDER.getPlayersStatLevel(player));
				for (int i = 0; i < items.size(); i++)
				{
					Integer roll = random.nextInt(10000);
					if (roll <= chances.get(i))
					{
						event.drops.add(items.get(i));
					}
				}
			}
		}
	}

	@ForgeSubscribe
	public void playerJoinWorld(EntityJoinWorldEvent event)
	{
		if (event.entity instanceof EntityPlayer)
		{
			EntityPlayer player = (EntityPlayer) event.entity;
			if (player.worldObj.isRemote)
			{
				PacketDispatcher.sendPacketToServer(PacketHandler.getStatSyncPacket(player, true));
			}
		}
	}

	@ForgeSubscribe
	public void playerFall(LivingFallEvent event)
	{
		if (event.entity instanceof EntityPlayer)
		{
			EntityPlayer player = (EntityPlayer) event.entity;
			int featherFallLevel = Stat.STAT_FEATHER_FALL.getPlayersStatLevel(player);
			if (event.distance < 3 + featherFallLevel * 0.1)
			{
				event.distance = 0;
			}
		}
	}

	@ForgeSubscribe
	public void playerDead(LivingDeathEvent event)
	{
		if (event.entityLiving instanceof EntityPlayer)
		{
			EntityPlayer player = (EntityPlayer) event.entityLiving;
			if (Stat.loseStatsOnDeath)
			{
				for (int stat = 0; stat < Stat.totalStats; stat++)
				{
					Helper.getPlayerPersistentNBT(player).setByte(Stat.stats[stat].key, (byte) 0);
				}
			}
		}
	}

	@ForgeSubscribe
	public void playerJump(LivingJumpEvent event)
	{
		if (event.entity instanceof EntityPlayer)
		{
			EntityPlayer player = (EntityPlayer) event.entity;
			if (player.isSprinting())
			{
				player.motionY *= 1 + Stat.STAT_LEAPERV.getBonus(player);
				player.motionX *= 1 + Stat.STAT_LEAPERH.getBonus(player);
				player.motionZ *= 1 + Stat.STAT_LEAPERH.getBonus(player);
			}
		}
	}

	@ForgeSubscribe
	public void playerBreakSpeed(PlayerEvent.BreakSpeed event)
	{
		ItemStack heldItem = event.entityPlayer.getHeldItem();
		EntityPlayer player = event.entityPlayer;
		Block block = event.block;
		int metadata = event.metadata;
		float multiplier = 1.0F + (Stat.STAT_MINING.isAffectedByStat(heldItem, block, metadata) * Stat.STAT_MINING.getBonus(player) + Stat.STAT_DIGGING.isAffectedByStat(heldItem, block, metadata) * Stat.STAT_DIGGING.getBonus(player) + Stat.STAT_CHOPPING.isAffectedByStat(heldItem, block, metadata) * Stat.STAT_CHOPPING.getBonus(player) + Stat.STAT_TRIMMING.isAffectedByStat(heldItem, block, metadata) * Stat.STAT_TRIMMING.getBonus(player));
		event.newSpeed = event.originalSpeed * multiplier;
		//	player.addChatMessage(heldItem.getDisplayName() + " multiplier: " + multiplier);
	}

	@ForgeSubscribe
	public void entityHurt(LivingHurtEvent event)
	{
		DamageSource source = event.source;
		if (event.entity instanceof EntityPlayer)
		{
			EntityPlayer player = (EntityPlayer) event.entity;
			float damageMultiplier = 1 - (Stat.STAT_PROTECTION.getAppliedBonus(player, source) + Stat.STAT_TOUGH_SKIN.getAppliedBonus(player, source) + Stat.STAT_FEATHER_FALL.getAppliedBonus(player, source) + Stat.STAT_TEMPERING.getAppliedBonus(player, source));
			event.ammount *= damageMultiplier;
			// Potential Knockback Resistance Stat
			/*
			Entity attacker = event.entityLiving;
			float knockbackModifier = EnchantmentHelper.getKnockbackModifier((EntityLivingBase) attacker, (EntityLivingBase) player);
			knockbackModifier += 1.5;
			float knockbackMultiplier = 1;
			knockbackModifier *= knockbackMultiplier;
			double velocityX = (-MathHelper.sin(attacker.rotationYaw * (float) Math.PI / 180.0F) * knockbackModifier * 0.5F);
			double velocityZ = (MathHelper.cos(attacker.rotationYaw * (float) Math.PI / 180.0F) * knockbackModifier * 0.5F);
			player.addVelocity(velocityX, 0, velocityZ);*/
		} else if (event.source.getEntity() instanceof EntityPlayer)
		{
			EntityPlayer player = (EntityPlayer) event.source.getEntity();
			ItemStack heldItem = player.getHeldItem();
			Entity target = event.entity;
			float damage = event.ammount;
			float bonus = 0;
			if (player.getHeldItem() != null)
			{
				bonus = Math.round(damage * (Stat.STAT_SWORDSMANSHIP.getAppliedBonus(player, heldItem) + Stat.STAT_BOWMANSHIP.getAppliedBonus(player, heldItem)));
			}
			event.ammount = bonus + damage;
			if (Stat.STAT_REAPER.isAffectedByStat(target) != 0)
			{
				float reap = Stat.STAT_REAPER.getBonus(player);
				float reapBonus = reap * Stat.STAT_STEALTH.getSecondaryBonus(player) / 100;
				float reapChance = (reap + reapBonus) * Stat.STAT_STEALTH.isAffectedByStat(player);
				if (player.getRNG().nextFloat() <= reapChance)
				{
					player.onEnchantmentCritical(target);
					player.worldObj.playSoundAtEntity(player, Reference.MOD_ID + ":reaper", 1.0f, 1.0f);
					event.ammount = 100000.0f;
				}
			}
		}
	}
}
