package goki.stats;

import goki.stats.lib.Reference;
import goki.stats.stats.Stat;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.util.DamageSource;
import net.minecraftforge.common.Configuration;

public abstract class DamageSourceProtectionStat extends Stat
{
	public List<String> damageSources = new ArrayList<String>();

	public DamageSourceProtectionStat(int id, String key, String name, int limit)
	{
		super(id, key, name, limit);
	}

	@Override
	public int isAffectedByStat(Object object)
	{
		if (object != null)
		{
			if (object instanceof DamageSource)
			{
				DamageSource source = (DamageSource) object;
				for (int i = 0; i < damageSources.size(); i++)
				{
					if (source.damageType.equals(damageSources.get(i)))
					{
						return 1;
					}
				}
			}
		}
		return 0;
	}

	@Override
	public void loadFromConfigurationFile(Configuration config)
	{
		damageSources.clear();
		String[] sources = Reference.configuration.get("Support", name + " Sources", getDefaultDamageSources()).getStringList();
		for (int i = 0; i < sources.length; i++)
		{
			damageSources.add(sources[i]);
		}
	}

	@Override
	public String toConfigurationString()
	{
		String configString = "";
		for (String s : damageSources)
		{
			configString += "," + s;
		}
		return configString.substring(1);
	}

	@Override
	public void saveToConfigurationFile(Configuration config)
	{
		String[] sources = new String[damageSources.size()];
		for (int i = 0; i < sources.length; i++)
		{
			sources[i] = damageSources.get(i);
		}
		Reference.configuration.get("Support", name + " Sources", getDefaultDamageSources()).set(sources);
	}

	@Override
	public void fromConfigurationString(String configString)
	{
		damageSources.clear();
		String[] configStringSplit = configString.split(",");
		for (String s : configStringSplit)
		{
			damageSources.add(s);
		}
	}

	public abstract String[] getDefaultDamageSources();
}
