package goki.stats;

import goki.stats.client.gui.GuiHandler;
import goki.stats.handlers.ClientPlayerTickHandler;
import goki.stats.handlers.CommonPlayerTickHandler;
import goki.stats.handlers.PacketHandler;
import goki.stats.handlers.ServerPlayerTickHandler;
import goki.stats.lib.Reference;
import goki.stats.stats.Stat;

import java.util.EnumSet;

import net.minecraft.command.ICommandManager;
import net.minecraft.command.ServerCommandManager;
import net.minecraft.server.MinecraftServer;
import net.minecraftforge.common.Configuration;
import net.minecraftforge.common.MinecraftForge;
import cpw.mods.fml.common.ITickHandler;
import cpw.mods.fml.common.Mod;
import cpw.mods.fml.common.Mod.EventHandler;
import cpw.mods.fml.common.Mod.Instance;
import cpw.mods.fml.common.SidedProxy;
import cpw.mods.fml.common.TickType;
import cpw.mods.fml.common.event.FMLInitializationEvent;
import cpw.mods.fml.common.event.FMLPostInitializationEvent;
import cpw.mods.fml.common.event.FMLPreInitializationEvent;
import cpw.mods.fml.common.event.FMLServerStartingEvent;
import cpw.mods.fml.common.network.NetworkMod;
import cpw.mods.fml.common.network.NetworkRegistry;
import cpw.mods.fml.common.registry.TickRegistry;
import cpw.mods.fml.relauncher.Side;

@Mod(modid = Reference.MOD_ID, name = Reference.MOD_NAME, version = Reference.VERSION)
@NetworkMod(clientSideRequired = true, serverSideRequired = true, channels =
{ Reference.SERVER_NETWORK_CHANNEL, Reference.CLIENT_NETWORK_CHANNEL, Reference.GENERIC_NETWORK_CHANNEL }, packetHandler = PacketHandler.class)
public class GokiStats
{

	@Instance("gokiStats")
	public static GokiStats instance;

	@SidedProxy(clientSide = "goki.stats.client.ClientProxy", serverSide = "goki.stats.CommonProxy")
	public static CommonProxy proxy;

	@EventHandler
	public void preInit(FMLPreInitializationEvent event)
	{
		instance = this;
		Reference.configuration = new Configuration(event.getSuggestedConfigurationFile());
		Reference.configuration.load();
		if (!Reference.configuration.get("Version", "Configuration Version", "v1").getString().equals(Reference.CONFIGURATION_VERSION))
		{
			System.err.println("gokiStats configuration file has changed! May cause errors! Delete the configuration file and relaunch.");
		}
		Stat.loadOptions(Reference.configuration);
		Stat.loadAllStatsFromConfiguration(Reference.configuration);

		proxy.registerKeybinding();
		proxy.registerHandlers();
	}

	@EventHandler
	public void init(FMLInitializationEvent event)
	{
		MinecraftForge.EVENT_BUS.register(new Events());
		MinecraftForge.EVENT_BUS.register(new CommonEvents());
		//GameRegistry.registerCraftingHandler(events);
		NetworkRegistry.instance().registerGuiHandler(this, new GuiHandler());
		TickRegistry.registerTickHandler(new ClientPlayerTickHandler(EnumSet.of(TickType.PLAYER)), Side.CLIENT);
		TickRegistry.registerTickHandler(new ServerPlayerTickHandler(EnumSet.of(TickType.PLAYER)), Side.SERVER);
		ITickHandler commonTick = new CommonPlayerTickHandler(EnumSet.of(TickType.PLAYER));
		TickRegistry.registerTickHandler(commonTick, Side.CLIENT);
		TickRegistry.registerTickHandler(commonTick, Side.SERVER);
	}

	@EventHandler
	public void postInit(FMLPostInitializationEvent event)
	{
		Stat.saveAllStatsToConfiguration(Reference.configuration);
		Stat.saveGlobalMultipliers(Reference.configuration);
		Reference.configuration.save();
	}

	@EventHandler
	public void serverStart(FMLServerStartingEvent event)
	{
		MinecraftServer server = MinecraftServer.getServer(); //Gets current server
		ICommandManager command = server.getCommandManager(); //Gets the command manager to use for server
		ServerCommandManager serverCommand = ((ServerCommandManager) command); //Turns it into another form to use
		serverCommand.registerCommand(new StatsCommand());
	}
}
