package goki.stats;

import goki.stats.lib.Reference;
import goki.stats.stats.Stat;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.Configuration;

public abstract class ToolSpecificStat extends Stat
{

	public List<ItemIdMetadataTuple> supportedItems = new ArrayList<ItemIdMetadataTuple>();

	public ToolSpecificStat(int id, String key, String name, int limit)
	{
		super(id, key, name, limit);
	}

	public abstract String getConfigurationKey();

	public abstract String[] getDefaultSupportedItems();

	public void addSupportForItem(ItemStack item)
	{
		loadFromConfigurationFile(Reference.configuration);
		if (item == null)
		{
			return;
		}
		boolean hasSubtypes = item.getHasSubtypes();
		int id = item.itemID;
		int meta = 0;

		if (hasSubtypes)
		{
			meta = item.getItemDamage();
		}
		ItemIdMetadataTuple iimt = new ItemIdMetadataTuple(id, meta);
		if (!supportedItems.contains(iimt))
		{
			supportedItems.add(iimt);
		}
		saveToConfigurationFile(Reference.configuration);
	}

	public void removeSupportForItem(ItemStack item)
	{
		if (item != null)
		{
			ItemIdMetadataTupleComparator iimtc = new ItemIdMetadataTupleComparator();
			loadFromConfigurationFile(Reference.configuration);

			ItemIdMetadataTuple iimt = new ItemIdMetadataTuple(item.itemID, 0);
			if (item.getHasSubtypes())
			{
				iimt.metadata = item.getItemDamage();
			}
			for (int i = 0; i < supportedItems.size(); i++)
			{
				ItemIdMetadataTuple ii = supportedItems.get(i);
				if (iimtc.compare(iimt, ii) == 1)
				{
					supportedItems.remove(ii);
					i--;
				}
			}

			saveToConfigurationFile(Reference.configuration);
		}
	}

	@Override
	public void loadFromConfigurationFile(Configuration config)
	{
		supportedItems.clear();
		String[] configStrings = Reference.configuration.get("Support", getConfigurationKey(), getDefaultSupportedItems()).getStringList();
		for (int i = 0; i < configStrings.length; i++)
		{
			supportedItems.add(new ItemIdMetadataTuple(configStrings[i]));
		}
	}

	@Override
	public String toConfigurationString()
	{
		String configString = "";
		for (int i = 0; i < supportedItems.size(); i++)
		{
			configString += "," + supportedItems.get(i).toConfigString();
		}
		return configString.substring(1);
	}

	@Override
	public void saveToConfigurationFile(Configuration config)
	{
		String[] toolIDs = new String[supportedItems.size()];
		for (int i = 0; i < toolIDs.length; i++)
		{
			toolIDs[i] = supportedItems.get(i).toConfigString();
		}
		Reference.configuration.get("Support", getConfigurationKey(), getDefaultSupportedItems()).set(toolIDs);
	}

	@Override
	public void fromConfigurationString(String configString)
	{
		supportedItems.clear();
		String[] configStringSplit = configString.split(",");
		for (int i = 0; i < configStringSplit.length; i++)
		{
			supportedItems.add(new ItemIdMetadataTuple(configStringSplit[i]));
		}
	}

	public boolean isItemSupported(ItemStack item)
	{
		for (int i = 0; i < supportedItems.size(); i++)
		{
			ItemIdMetadataTuple iimt = supportedItems.get(i);
			if (item.itemID == iimt.id)
			{
				if (item.getHasSubtypes() && item.getItemDamage() == iimt.metadata)
				{
					System.out.println("Item is supported!");
					return true;
				} else if (!item.getHasSubtypes())
				{
					System.out.println("Item is supported!");
					return true;
				}
			}
		}

		return false;
	}

	@Override
	public int isAffectedByStat(Object object)
	{
		if (object != null && object instanceof ItemStack)
		{
			ItemStack item = (ItemStack) object;
			if (isItemSupported(item))
			{
				System.out.println("Item is affected by stat!");
				return 1;
			}
		}
		return 0;
	}

	@Override
	public float getAppliedBonus(EntityPlayer player, Object object)
	{
		return getBonus(player) * isAffectedByStat(object);
	}

}
