package goki.stats.client.gui;

import goki.stats.handlers.PacketHandler;
import goki.stats.lib.Helper;
import goki.stats.lib.Reference;
import goki.stats.stats.Stat;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.gui.GuiScreen;
import net.minecraft.entity.player.EntityPlayer;

import org.lwjgl.util.vector.Vector2f;

import cpw.mods.fml.common.network.PacketDispatcher;

public class GuiStats extends GuiScreen
{

	private EntityPlayer player = null;
	public static final int STATUS_BUTTON_WIDTH = 24;
	public static final int STATUS_BUTTON_HEIGHT = 24;
	public static float SCALE = 1.0f;
	private static final int HORIZONTAL_SPACING = 8;
	private static final int VERTICAL_SPACING = 12;
	//private static final int COLUMNS = 5;
	public static final int IMAGE_ROWS = 10;
	private static final int[] COLUMNS = new int[]
	{ 4, 3, 5, 3, 4 };
	private int currentColumn = 0;
	private int currentRow = 0;
	private GuiStatTooltip toolTip = null;

	private final String noHoverHeaderMessage = "Stats";
	private final String noHoverSubMessage = "Hover over an icon for more information.";

	private String headerMessage = "";
	private String subMessage = "";

	public GuiStats(EntityPlayer player)
	{
		this.player = player;
	}

	@Override
	public void drawScreen(int mouseX, int mouseY, float par3)
	{
		int ttx = 0;
		int tty = 0;
		toolTip = null;
		this.drawDefaultBackground();
		super.drawScreen(mouseX, mouseY, par3);
		for (int i = 0; i < buttonList.size(); i++)
		{
			if (buttonList.get(i) instanceof GuiStatButton)
			{
				GuiStatButton button = (GuiStatButton) buttonList.get(i);
				if (button.isUnderMouse(mouseX, mouseY))
				{
					toolTip = new GuiStatTooltip(Stat.stats[i], player);
					ttx = button.xPosition + STATUS_BUTTON_WIDTH / 2;
					tty = button.yPosition - 1;
					break;
				}
			}
		}
		drawCenteredString(fontRenderer, "Current XP: " + Helper.getXPTotal(player.experienceLevel, player.experience) + "xp", width / 2, this.height - 16, 0xFFFFFFFF);

		if (toolTip != null)
			toolTip.draw(ttx, tty, 0);

	}

	@SuppressWarnings("unchecked")
	@Override
	public void initGui()
	{
		for (int stat = 0; stat < Stat.totalStats; stat++)
		{
			Vector2f pos = this.getButtonPosition(stat);
			this.buttonList.add(new GuiStatButton(stat, (int) pos.x, (int) pos.y, STATUS_BUTTON_WIDTH, STATUS_BUTTON_HEIGHT, Stat.stats[stat], player));
			currentColumn++;
			if (currentColumn >= COLUMNS[currentRow])
			{
				currentRow++;
				currentColumn = 0;
			}
			if (currentRow >= COLUMNS.length)
			{
				currentRow = COLUMNS.length - 1;
			}
		}
	}

	private Vector2f getButtonPosition(int n)
	{
		Vector2f vec = new Vector2f();
		int columns = COLUMNS[currentRow];
		int x = n % columns;
		int y = currentRow;
		int rows = COLUMNS.length;
		int amount = columns;
		float width = (amount * (STATUS_BUTTON_WIDTH + HORIZONTAL_SPACING)) * SCALE;
		float height = (rows * (STATUS_BUTTON_HEIGHT + VERTICAL_SPACING)) * SCALE;
		vec.x = width / amount * x + (this.width - width + HORIZONTAL_SPACING) / 2;
		vec.y = height / rows * y + (this.height - height + VERTICAL_SPACING) / 2;
		return vec;
	}

	@Override
	protected void actionPerformed(GuiButton button)
	{
		if (button.id >= 0 && button.id <= Stat.totalStats)
		{
			PacketDispatcher.sendPacketToServer(PacketHandler.getStatPacket(player, Reference.ALTER_STAT_PACKET_ID, button.id, 1));
			PacketDispatcher.sendPacketToServer(PacketHandler.getStatSyncPacket(player, true));
		}
	}

	@Override
	public boolean doesGuiPauseGame()
	{
		return false;
	}

}
