package goki.stats.handlers;

import goki.stats.stats.Stat;

import java.util.EnumSet;
import java.util.UUID;

import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.ai.attributes.AttributeInstance;
import net.minecraft.entity.ai.attributes.AttributeModifier;
import net.minecraft.entity.player.EntityPlayer;
import cpw.mods.fml.common.ITickHandler;
import cpw.mods.fml.common.TickType;

public class CommonPlayerTickHandler implements ITickHandler
{
	private final EnumSet<TickType> ticksToGet;

	public final static UUID knockbackResistanceID = UUID.randomUUID();
	public final static UUID stealthSpeedID = UUID.randomUUID();

	public CommonPlayerTickHandler(EnumSet<TickType> ticksToGet)
	{
		this.ticksToGet = ticksToGet;
	}

	@Override
	public EnumSet<TickType> ticks()
	{
		return ticksToGet;
	}

	@Override
	public String getLabel()
	{
		return "gokiStatsTick";
	}

	public static void playerTick(EntityPlayer player)
	{
		if (player.isInWater())
		{
			float multiplier = Stat.STAT_ATHLETICISM.getBonus(player);
			float multiplierY = multiplier;
			if (player.motionY < 0)
			{
				multiplierY = 0;
			}
			player.moveEntity(player.motionX * multiplier, player.motionY * multiplierY, player.motionZ * multiplier);
		}
		if (player.isOnLadder() && !player.isSneaking() && player.isCollidedHorizontally)
		{
			float multiplier = Stat.STAT_CLIMBING.getBonus(player);
			player.moveEntity(player.motionX, player.motionY * multiplier, player.motionZ);
		}

		AttributeInstance atinst;
		AttributeModifier mod;

		/* Movement speed bonus when sneaking */
		atinst = player.getEntityAttribute(SharedMonsterAttributes.movementSpeed);
		mod = new AttributeModifier(stealthSpeedID, "SneakSpeed", Stat.STAT_STEALTH.getBonus(player) / 100, 1);
		if (player.isSneaking())
		{
			if (atinst.getModifier(stealthSpeedID) == null)
			{
				atinst.applyModifier(mod);
			}
		} else
		{

			if (atinst.getModifier(stealthSpeedID) != null)
			{
				atinst.removeModifier(mod);
			}
		}

		/* Knockback Resistance when blocking with weapon */
		atinst = player.getEntityAttribute(SharedMonsterAttributes.knockbackResistance);
		mod = new AttributeModifier(knockbackResistanceID, "KnockbackResistance", Stat.STAT_STEADY_GUARD.getBonus(player), 0);
		if (player.isBlocking())
		{
			if (atinst.getModifier(knockbackResistanceID) == null)
			{
				atinst.applyModifier(mod);
			}
		} else
		{

			if (atinst.getModifier(knockbackResistanceID) != null)
			{
				atinst.removeModifier(mod);
			}
		}
	}

	@Override
	public void tickStart(EnumSet<TickType> type, Object... tickData)
	{
		playerTick((EntityPlayer) tickData[0]);
	}

	@Override
	public void tickEnd(EnumSet<TickType> type, Object... tickData)
	{
	}
}