package goki.stats.handlers;

import goki.stats.lib.Helper;
import goki.stats.lib.Reference;
import goki.stats.stats.Stat;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import net.minecraft.client.entity.EntityClientPlayerMP;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.INetworkManager;
import net.minecraft.network.packet.Packet250CustomPayload;
import cpw.mods.fml.common.network.IPacketHandler;
import cpw.mods.fml.common.network.PacketDispatcher;
import cpw.mods.fml.common.network.Player;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

public class PacketHandler implements IPacketHandler
{

	@Override
	public void onPacketData(INetworkManager manager, Packet250CustomPayload packet, Player playerEntity)
	{
		if (packet.channel.equals(Reference.SERVER_NETWORK_CHANNEL))
		{
			handleServerRpgPacket(packet, playerEntity);
		} else if (packet.channel.equals(Reference.CLIENT_NETWORK_CHANNEL))
		{
			handleClientRpgPacket(packet, playerEntity);
		} else if (packet.channel.equals(Reference.GENERIC_NETWORK_CHANNEL))
		{
			handleGenericPacket(packet, playerEntity);
		}
	}

	private void handleGenericPacket(Packet250CustomPayload packet, Player playerEntity)
	{
		String type = "";
		DataInputStream inputStream = new DataInputStream(new ByteArrayInputStream(packet.data));
		try
		{
			type = inputStream.readUTF();
		} catch (IOException e)
		{
			e.printStackTrace();
			return;
		}
		if (type.equals("syncXP"))
		{
			EntityPlayer player = (EntityPlayer) playerEntity;

			try
			{
				float percent = inputStream.readFloat();
				int level = inputStream.readInt();
				int total = inputStream.readInt();
				player.experience = percent;
				player.experienceLevel = level;
				player.experienceTotal = total;
			} catch (IOException e)
			{
				e.printStackTrace();
				return;
			}
		} else if (type.equals("syncStatConfig"))
		{
			EntityPlayer player = (EntityPlayer) playerEntity;

			try
			{
				boolean deathLoss = inputStream.readBoolean();
				float newBonus = inputStream.readFloat();
				float newCost = inputStream.readFloat();
				float newLimit = inputStream.readFloat();
				Stat.loseStatsOnDeath = deathLoss;
				Stat.globalBonusMultiplier = newBonus;
				Stat.globalCostMultiplier = newCost;
				Stat.globalLimitMultiplier = newLimit;
				for (int i = 0; i < Stat.stats.length; i++)
				{
					if (Stat.stats[i] != null)
					{
						String utf = inputStream.readUTF();
						Stat.stats[i].fromConfigurationString(utf);
					}
				}
			} catch (IOException e)
			{
				e.printStackTrace();
				return;
			}
		}
	}

	private void handleServerRpgPacket(Packet250CustomPayload packet, Player playerEntity)
	{
		EntityPlayerMP playerMP = (EntityPlayerMP) playerEntity;
		DataInputStream inputStream = new DataInputStream(new ByteArrayInputStream(packet.data));

		int packetType = 0;
		int operation = 0;
		int amount = 0;

		try
		{
			packetType = inputStream.readInt();
		} catch (IOException e)
		{
			e.printStackTrace();
			return;
		}
		if (packetType == Reference.SYNC_STATS_PACKET_ID)
		{
			PacketDispatcher.sendPacketToPlayer(getStatSyncPacket(playerMP, false), playerEntity);
		} else
		{
			try
			{
				operation = inputStream.readInt();
			} catch (IOException e)
			{
				e.printStackTrace();
				return;
			}
			if (packetType == Reference.CHECK_STAT_PACKET_ID)
			{
				playerMP.addChatMessage(Stat.stats[operation].name + " Level: " + playerMP.getEntityData().getByte(Stat.stats[operation].key));
			} else if (packetType == Reference.ALTER_STAT_PACKET_ID)
			{
				try
				{
					amount = inputStream.readInt();
				} catch (IOException e)
				{
					e.printStackTrace();
					return;
				}
				Stat stat = Stat.stats[operation];
				String statKey = stat.key;
				int level = stat.getPlayersStatLevel(playerMP);
				int cost = stat.getCost(level);
				int currentXP = Helper.getXPTotal(playerMP.experienceLevel, playerMP.experience);

				if (Stat.stats[operation].enabled)
				{
					if (level < Stat.stats[operation].getLimit())
					{
						if (currentXP >= cost)
						{
							NBTTagCompound nbt = Helper.getPlayerPersistentNBT(playerMP);
							nbt.setByte(statKey, (byte) (level + amount));
							Helper.setPlayersExpTo(playerMP, currentXP - cost);
							PacketDispatcher.sendPacketToPlayer(getXpSyncPacket(playerMP.experience, playerMP.experienceLevel, playerMP.experienceTotal), playerEntity);
						}
					} else if (level > Stat.stats[operation].getLimit())
					{
						NBTTagCompound nbt = Helper.getPlayerPersistentNBT(playerMP);
						nbt.setByte(statKey, (byte) Stat.stats[operation].getLimit());
					}
				}
			}
		}
	}

	@SideOnly(Side.CLIENT)
	private void handleClientRpgPacket(Packet250CustomPayload packet, Player playerEntity)
	{
		EntityClientPlayerMP playerMP = (EntityClientPlayerMP) playerEntity;
		DataInputStream inputStream = new DataInputStream(new ByteArrayInputStream(packet.data));

		int packetType = 0;

		try
		{
			packetType = inputStream.readInt();
		} catch (IOException e)
		{
			e.printStackTrace();
			return;
		}

		if (packetType == Reference.SYNC_STATS_PACKET_ID)
		{
			for (int stat = 0; stat < Stat.totalStats; stat++)
			{
				try
				{
					NBTTagCompound nbt = Helper.getPlayerPersistentNBT(playerMP);
					nbt.setByte(Stat.stats[stat].key, (byte) inputStream.readInt());
				} catch (IOException e)
				{
					e.printStackTrace();
					return;
				}
			}
		}
	}

	public static Packet250CustomPayload getStatSyncPacket(EntityPlayer player, boolean server)
	{
		Packet250CustomPayload packet;
		if (server)
		{
			ByteArrayOutputStream bos = new ByteArrayOutputStream(4);
			DataOutputStream outputStream = new DataOutputStream(bos);
			try
			{
				outputStream.writeInt(Reference.SYNC_STATS_PACKET_ID);
			} catch (Exception ex)
			{
				ex.printStackTrace();
			}

			packet = new Packet250CustomPayload();
			packet.channel = Reference.SERVER_NETWORK_CHANNEL;
			packet.data = bos.toByteArray();
			packet.length = bos.size();
		} else
		{
			int size = 0;
			for (int stat = 0; stat < Stat.totalStats; stat++)
			{
				size += 4;
			}
			ByteArrayOutputStream bos = new ByteArrayOutputStream(size + 4);
			DataOutputStream outputStream = new DataOutputStream(bos);
			try
			{
				outputStream.writeInt(Reference.SYNC_STATS_PACKET_ID);
				for (int stat = 0; stat < Stat.totalStats; stat++)
				{
					outputStream.writeInt((int) Helper.getPlayerPersistentNBT(player).getByte(Stat.stats[stat].key));
				}
			} catch (Exception ex)
			{
				ex.printStackTrace();
			}

			packet = new Packet250CustomPayload();
			packet.channel = Reference.CLIENT_NETWORK_CHANNEL;
			packet.data = bos.toByteArray();
			packet.length = bos.size();
		}
		return packet;
	}

	public static Packet250CustomPayload getStatPacket(EntityPlayer player, int type, int operation)
	{
		ByteArrayOutputStream bos = new ByteArrayOutputStream(8);
		DataOutputStream outputStream = new DataOutputStream(bos);
		try
		{
			outputStream.writeInt(type);
			outputStream.writeInt(operation);
		} catch (Exception ex)
		{
			ex.printStackTrace();
		}

		Packet250CustomPayload packet = new Packet250CustomPayload();
		packet.channel = Reference.SERVER_NETWORK_CHANNEL;
		packet.data = bos.toByteArray();
		packet.length = bos.size();

		return packet;
	}

	public static Packet250CustomPayload getStatPacket(EntityPlayer player, int type, int operation, int amount)
	{
		ByteArrayOutputStream bos = new ByteArrayOutputStream(12);
		DataOutputStream outputStream = new DataOutputStream(bos);
		try
		{
			outputStream.writeInt(type);
			outputStream.writeInt(operation);
			outputStream.writeInt(amount);
		} catch (Exception ex)
		{
			ex.printStackTrace();
		}

		Packet250CustomPayload packet = new Packet250CustomPayload();
		packet.channel = Reference.SERVER_NETWORK_CHANNEL;
		packet.data = bos.toByteArray();
		packet.length = bos.size();

		return packet;
	}

	public static Packet250CustomPayload getGenericPacket(String type, byte[] additionalData)
	{
		ByteArrayOutputStream bos = new ByteArrayOutputStream(16);
		DataOutputStream outputStream = new DataOutputStream(bos);
		try
		{
			outputStream.writeUTF(type);
		} catch (Exception ex)
		{
			ex.printStackTrace();
		}

		if (additionalData != null)
		{
			try
			{
				outputStream.write(additionalData);
			} catch (Exception ex)
			{
				ex.printStackTrace();
			}
		}

		Packet250CustomPayload packet = new Packet250CustomPayload();
		packet.channel = Reference.GENERIC_NETWORK_CHANNEL;
		packet.data = bos.toByteArray();
		packet.length = bos.size();

		return packet;
	}

	public static Packet250CustomPayload getXpSyncPacket(float experience, int experienceLevel, int experienceTotal)
	{
		ByteArrayOutputStream bos = new ByteArrayOutputStream(32);
		DataOutputStream outputStream = new DataOutputStream(bos);
		try
		{
			outputStream.writeFloat(experience);
			outputStream.writeInt(experienceLevel);
			outputStream.writeInt(experienceTotal);
		} catch (Exception ex)
		{
			ex.printStackTrace();
		}
		return getGenericPacket("syncXP", bos.toByteArray());
	}

	public static Packet250CustomPayload getStatConfigPacket(Stat stat)
	{
		ByteArrayOutputStream bos = new ByteArrayOutputStream(1);
		DataOutputStream outputStream = new DataOutputStream(bos);
		try
		{
			outputStream.writeBoolean(stat.loseStatsOnDeath);
			outputStream.writeFloat(stat.globalBonusMultiplier);
			outputStream.writeFloat(stat.globalCostMultiplier);
			outputStream.writeFloat(stat.globalLimitMultiplier);
			for (int i = 0; i < Stat.stats.length; i++)
			{
				if (Stat.stats[i] != null)
				{
					outputStream.writeUTF(Stat.stats[i].toConfigurationString());
				}
			}

		} catch (Exception ex)
		{
			ex.printStackTrace();
		}
		return getGenericPacket("syncStatConfig", bos.toByteArray());
	}
}