package goki.stats.lib;

import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.potion.Potion;
import net.minecraft.potion.PotionEffect;
import net.minecraft.util.DamageSource;
import net.minecraft.util.MovingObjectPosition;

public class Helper
{

	public static NBTTagCompound getPlayerPersistentNBT(EntityPlayer player)
	{
		NBTTagCompound nbt;
		if (!player.getEntityData().hasKey(EntityPlayer.PERSISTED_NBT_TAG))
		{
			nbt = new NBTTagCompound();
			player.getEntityData().setCompoundTag(EntityPlayer.PERSISTED_NBT_TAG, nbt);
		}
		nbt = player.getEntityData().getCompoundTag(EntityPlayer.PERSISTED_NBT_TAG);
		return nbt;
	}

	public static float trimDecimals(float in, int decimals)
	{
		float f = (float) (in * Math.pow(10, decimals));
		int i = (int) f;
		return i / (float) Math.pow(10, decimals);
	}

	public static void setPlayersExpTo(EntityPlayer player, int total)
	{
		player.experienceLevel = getLevelFromXPValue(total);
		player.experience = getCurrentFromXPValue(total);
	}

	public static int getXPTotal(int xpLevel, float current)
	{
		return (int) (getXPValueFromLevel(xpLevel) + getXPValueToNextLevel(xpLevel) * current);
	}

	public static int getXPTotal(EntityPlayer player)
	{
		return (int) (getXPValueFromLevel(player.experienceLevel) + getXPValueToNextLevel(player.experienceLevel) * player.experience);
	}

	public static int getLevelFromXPValue(int value)
	{
		int level = 0;
		if (value >= getXPValueFromLevel(30))
		{
			level = (int) ((1.0 / 14.0) * (Math.sqrt(56.0 * value - 32511.0) + 303.0));
		} else if (value >= getXPValueFromLevel(15))
		{
			level = (int) ((1.0 / 6.0) * (Math.sqrt(24.0 * value - 5159.0) + 59.0));
		} else
		{
			level = (int) (value / 17.0);
		}
		return level;
	}

	public static float getCurrentFromXPValue(int value)
	{
		if (value == 0)
		{
			return 0;
		}
		int level = getLevelFromXPValue(value);
		int needed = getXPValueFromLevel(level);
		int next = getXPValueToNextLevel(level);
		int difference = value - needed;
		float current = (float) difference / next;
		return current;
	}

	public static int getXPValueFromLevel(int xpLevel)
	{
		int val = 0;
		if (xpLevel >= 30)
		{
			val = (int) (3.5 * Math.pow(xpLevel, 2) - 151.5 * xpLevel + 2220);
		} else if (xpLevel >= 15)
		{
			val = (int) (1.5 * Math.pow(xpLevel, 2) - 29.5 * xpLevel + 360);
		} else
		{
			val = 17 * xpLevel;
		}
		return val;
	}

	public static int getXPValueToNextLevel(int xpLevel)
	{
		int val = 0;
		if (xpLevel >= 30)
		{
			val = 7 * xpLevel - 148;
		} else if (xpLevel >= 15)
		{
			val = 3 * xpLevel - 28;
		} else
		{
			val = 17;
		}

		return val;
	}

	public static int[] getBlockPositionInfrontOfRaycastedBlock(EntityPlayer player, int distance)
	{
		MovingObjectPosition mop = player.rayTrace(distance, 1.0f);
		if (mop != null)
		{
			int rcx = mop.blockX;
			int rcy = mop.blockY;
			int rcz = mop.blockZ;
			switch (mop.sideHit)
			{
			case 0:
				rcy--;
				break;
			case 1:
				rcy++;
				break;
			case 2:
				rcz--;
				break;
			case 3:
				rcz++;
				break;
			case 4:
				rcx--;
				break;
			case 5:
				rcx++;
				break;
			}
			return new int[]
			{ rcx, rcy, rcz };
		}
		return null;
	}

	public static float getDamageDealt(EntityPlayer player, Entity target, DamageSource source)
	{
		float damage = (float) player.getEntityAttribute(SharedMonsterAttributes.attackDamage).getAttributeValue();
		float bonusDamage = 0;
		boolean targetIsLiving = target instanceof EntityLivingBase;
		boolean critical = false;
		if (targetIsLiving)
		{
			bonusDamage = EnchantmentHelper.getEnchantmentModifierLiving(player, (EntityLivingBase) target);
		}
		if (damage > 0 || bonusDamage > 0.0F)
		{
			critical = player.fallDistance > 0.0F && !player.onGround && !player.isOnLadder() && !player.isInWater() && !player.isPotionActive(Potion.blindness) && player.ridingEntity == null && targetIsLiving;
			if (critical && damage > 0)
			{
				damage *= 1.5f;
			}
			damage += bonusDamage;
		}
		return damage;
	}

	public static float getFallResistance(EntityLivingBase entity)
	{
		float resistance = 3.0f;
		PotionEffect potioneffect = entity.getActivePotionEffect(Potion.jump);
		float bonus = potioneffect != null ? (float) (potioneffect.getAmplifier() + 1) : 0.0F;
		return resistance + bonus;
	}

}
