package goki.stats.stats;

import goki.stats.DamageSourceProtectionStat;
import goki.stats.ToolSpecificStat;
import goki.stats.lib.Helper;
import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.Configuration;
import net.minecraftforge.common.ForgeHooks;

public class Stat implements IStat
{
	public static boolean loseStatsOnDeath = true;
	public static float globalCostMultiplier = 1.0f;
	public static float globalLimitMultiplier = 2.5f;
	public static float globalBonusMultiplier = 1.0f;
	public static Stat[] stats = new Stat[32];
	public static int totalStats = 0;

	public static final ToolSpecificStat STAT_MINING = new StatMining(0, "grpg_Mining", "Mining Proficiency", 10);
	public static final ToolSpecificStat STAT_DIGGING = new StatDigging(1, "grpg_Digging", "Digging Proficiency", 10);
	public static final ToolSpecificStat STAT_CHOPPING = new StatChopping(2, "grpg_Chopping", "Chopping Proficiency", 10);
	public static final ToolSpecificStat STAT_TRIMMING = new StatTrimming(3, "grpg_Trimming", "Trimming Proficiency", 10);
	public static final DamageSourceProtectionStat STAT_PROTECTION = new StatProtection(4, "grpg_Protection", "Protection", 10);
	public static final DamageSourceProtectionStat STAT_TEMPERING = new StatTempering(5, "grpg_Tempering", "Tempering", 10);
	public static final DamageSourceProtectionStat STAT_TOUGH_SKIN = new StatToughSkin(6, "grpg_ToughSkin", "Tough Skin", 10);
	public static final DamageSourceProtectionStat STAT_FEATHER_FALL = new StatFeatherFall(7, "grpg_FeatherFall", "Feather Fall", 10);
	public static final Stat STAT_LEAPERH = new StatLeaperH(8, "grpg_LeaperH", "LeaperH", 10);
	public static final Stat STAT_LEAPERV = new StatLeaperV(9, "grpg_LeaperV", "LeaperV", 10);
	public static final Stat STAT_ATHLETICISM = new StatAthleticism(10, "grpg_Athleticism", "Athleticism", 10);
	public static final Stat STAT_CLIMBING = new StatClimbing(11, "grpg_Climbing", "Climbing", 10);
	//public static final Stat STAT_PUGILISM = new StatPugilism(12, "grpg_Pugilism", "Pugilism", 10);
	public static final ToolSpecificStat STAT_SWORDSMANSHIP = new StatSwordsmanship(13, "grpg_Swordsmanship", "Swordsmanship", 10);
	public static final ToolSpecificStat STAT_BOWMANSHIP = new StatBowmanship(14, "grpg_Bowmanship", "Bowmanship", 10);
	public static final Stat STAT_REAPER = new StatReaper(15, "grpg_Reaper", "Reaper", 10);
	public static final Stat STAT_TREASURE_FINDER = new StatTreasureFinder(16, "grpg_Treasure_Finder", "Treasure Finder", 3);
	public static final Stat STAT_FURNACE_FINESSE = new StatFurnaceFinesse(17, "grpg_Furnace_Finesse", "Furnace Finesse", 10);
	public static final Stat STAT_STEADY_GUARD = new StatSteadyGuard(18, "grpg_Steady_Guard", "Steady Guard", 10);
	public static final Stat STAT_STEALTH = new StatStealth(19, "grpg_Stealth", "Stealth", 10);
	//public static final Stat STAT_FOCUS = new StatFocus(14, "grpg_Focus", "Focus", 25);

	public int imageID;
	private int limit;
	public String key;
	public String name;
	public float costMultiplier = 1.0f;
	public float limitMultiplier = 1.0f;
	public float bonusMultiplier = 1.0f;
	public boolean enabled = true;

	public Stat()
	{
		this.imageID = -1;
		this.limit = 0;
		this.key = "Dummy";
		this.name = "Dummy Stat";
	}

	public Stat(int id, String key, String name, int limit)
	{
		this.imageID = id;
		this.limit = limit;
		this.key = key;
		this.name = name;
		stats[totalStats++] = this;
	}

	public static void loadOptions(Configuration config)
	{
		globalCostMultiplier = (float) config.get("Global Modifiers", "Cost Muliplier", 1.0, "A flat multiplier on the cost to upgrade all stats.").getDouble(1.0);
		globalLimitMultiplier = (float) config.get("Global Modifiers", "Limit Muliplier", 2.5, "A flat multiplier on the level limit of all stats.").getDouble(1.0);
		globalBonusMultiplier = (float) config.get("Global Modifiers", "Bonus Muliplier", 1.0, "A flat multiplier on the bonus all stats gives.").getDouble(1.0);
		loseStatsOnDeath = config.get("Options", "Death Loss", true, "Lose stats on death?").getBoolean(true);
	}

	public static void saveGlobalMultipliers(Configuration config)
	{
		config.get("Global Modifiers", "Cost Muliplier", 1.0, "A flat multiplier on the cost to upgrade all stats.").set(globalCostMultiplier);
		config.get("Global Modifiers", "Limit Muliplier", 2.5, "A flat multiplier on the level limit of all stats.").set(globalLimitMultiplier);
		config.get("Global Modifiers", "Bonus Muliplier", 1.0, "A flat multiplier on the bonus all stats gives.").set(globalBonusMultiplier);
		config.get("Options", "Death Loss", true, "Lose stats on death?").set(loseStatsOnDeath);
	}

	@Override
	public float getBonus(int level)
	{
		return 0;
	}

	@Override
	public float getBonus(EntityPlayer player)
	{
		return getBonus(getPlayersStatLevel(player));
	}

	@Override
	public int isAffectedByStat(Object object)
	{
		return 0;
	}

	@Override
	public String getSimpleDescriptionString()
	{
		return null;
	}

	@Override
	public String getAppliedDescriptionString(EntityPlayer player)
	{
		return null;
	}

	public static int getPlayersStatLevel(EntityPlayer player, Stat stat)
	{
		return Helper.getPlayerPersistentNBT(player).getByte(stat.key);
	}

	public int getPlayersStatLevel(EntityPlayer player)
	{
		return Helper.getPlayerPersistentNBT(player).getByte(this.key);
	}

	@Override
	public int getCost(int level)
	{
		return (int) ((Math.pow(level, 1.60) + 6 + level) * globalCostMultiplier);
	}

	public static float getFinalBonus(float currentBonus)
	{
		return currentBonus * globalBonusMultiplier;
	}

	@Override
	public float getSecondaryBonus(int level)
	{
		return 0;
	}

	@Override
	public float getSecondaryBonus(EntityPlayer player)
	{
		return getSecondaryBonus(getPlayersStatLevel(player));
	}

	public void loadFromConfigurationFile(Configuration config)
	{
	}

	public void fromConfigurationString(String configString)
	{
	}

	public void saveToConfigurationFile(Configuration config)
	{
	}

	public String toConfigurationString()
	{
		return "!";
	}

	public static void loadAllStatsFromConfiguration(Configuration config)
	{
		for (int i = 0; i < totalStats; i++)
		{
			stats[i].loadFromConfigurationFile(config);
		}
	}

	public static void saveAllStatsToConfiguration(Configuration config)
	{
		for (int i = 0; i < totalStats; i++)
		{
			stats[i].saveToConfigurationFile(config);
		}
	}

	@Override
	public float getAppliedBonus(EntityPlayer player, Object object)
	{
		return getBonus(player) * isAffectedByStat(object);
	}

	@Override
	public int isAffectedByStat(Object object1, Object object2)
	{
		return isAffectedByStat(object1);
	}

	// Default for stats looking for a itemstack, block, and metadata when checking if affected
	@Override
	public int isAffectedByStat(Object object1, Object object2, Object object3)
	{
		if (object1 instanceof ItemStack && object2 instanceof Block && object3 instanceof Integer)
		{
			ItemStack stack = (ItemStack) object1;
			Block block = (Block) object2;
			int metadata = (Integer) object3;
			if (ForgeHooks.isToolEffective(stack, block, metadata))
			{
				return 1;
			}
		}
		return isAffectedByStat(object1);
	}

	@Override
	public int getLimit()
	{
		if (globalLimitMultiplier <= 0)
		{
			return Byte.MAX_VALUE;
		}
		return (int) (limit * globalLimitMultiplier);
	}

}
